﻿using System;
using System.IO;
using System.Diagnostics;
using System.Globalization;
using System.Runtime.InteropServices;
using System.ComponentModel.Design;
using Microsoft.Win32;
using Microsoft.VisualStudio;
using Microsoft.VisualStudio.Shell.Interop;
using Microsoft.VisualStudio.OLE.Interop;
using Microsoft.VisualStudio.Shell;

namespace Microsoft.Samples.VisualStudio.BrowseWithExplorer
{
    /// <summary>
    /// This is the class that implements the package exposed by this assembly.
    ///
    /// The minimum requirement for a class to be considered a valid package for Visual Studio
    /// is to implement the IVsPackage interface and register itself with the shell.
    /// This package uses the helper classes defined inside the Managed Package Framework (MPF)
    /// to do it: it derives from the Package class that provides the implementation of the 
    /// IVsPackage interface and uses the registration attributes defined in the framework to 
    /// register itself and its components with the shell.
    /// </summary>
    // This attribute tells the PkgDef creation utility (CreatePkgDef.exe) that this class is
    // a package.
    [PackageRegistration(UseManagedResourcesOnly = true)]
    // This attribute is used to register the information needed to show this package
    // in the Help/About dialog of Visual Studio.
    [InstalledProductRegistration("#110", "#112", "1.0", IconResourceID = 400)]
    // Declare this pacakge will add standard previewers
    [ProvideStandardPreviewer(GuidList.guidBrowseWithExplorerPkgString, "Internet Explorer (Kiosk, InPrivate) Previewers")]
    // The package Guid
    [Guid(GuidList.guidBrowseWithExplorerPkgString)]
    public sealed class BrowseWithExplorerPackage : Package, IVsPackage
    {
        /// <summary>
        /// Called by shell to register previewers, reset toolbox items, etc
        /// 
        /// NOTE:  
        /// Although the Package class inside the Managed Package Framework (MPF) from which we derive 
        /// implemens the IVsPackage interface and the ResetDefaults function, at least in VS2012 and 
        /// older versions it does not call virtual functions for PKGRF_ADDSTDPREVIEWER flags value.
        /// (it only does it for toolbox-related settings, by raising ToolboxInitialized and ToolboxUpgraded). 
        /// Therefore our pacakge class needs to reimplement IVsPackage in order to override ResetDefaults directly.
        /// </summary>
        /// <param name="grfFlags"></param>
        /// <returns></returns>
        int IVsPackage.ResetDefaults(uint grfFlags)
        {
            // Override the base class implementation and register the previewers supported by this package
            if (grfFlags == (uint)__VSPKGRESETFLAGS.PKGRF_ADDSTDPREVIEWER)
            {
                // NOTE: When registering a standard previewer, the first argument in the AddStandardPreviewer, although it's called pszExePath
                // it also allows specifying command line arguments for the previewer in addition to the path to the browser's executable.
                // If there are no quotes in the path, the whole string is assumed to be the full path to the binary exe file of the previewer.
                // However, if quotes are used, arguments can be added separated by a space from the quoted filename. %1, %URL or %url can be
                // used in the arguments list, and Visual Studio will replace it at runtime with the name of the file to be previewed
                // (as a URL-formatted quoted string, e.g. "file:///C:/temp/My%20File.txt" ).
                // If the arguments don't contain either of %1, %URL, %url, then Visual Studio will append the file name to be previewed to the 
                // pszExePath string before launching the previewer process with that concatenated string as arguments.
                // Multiple previewers can be registered with the same executable path if the list of arguments passed are different.

                // Per http://msdn.microsoft.com/en-us/library/ee330728(v=VS.85).aspx
                // "IExplore -k filename" opens the file in Kiosk mode, with no toolbars, menus, etc (close it with Alt-F4)
                // "IExplore -private filename" opens the file using InPrivate mode
                string programFiles = Environment.GetFolderPath(Environment.SpecialFolder.ProgramFiles);
                string explorerExe = Path.Combine(programFiles, "Internet Explorer", "iexplore.exe");
                string cmdLineKiosk = String.Format("\"{0}\" -k %1", explorerExe); // here we specify the exact location in the command line where we want the previewed file name to be placed
                string cmdLineInPrivate = String.Format("\"{0}\" -private", explorerExe); // here we let Visual Studio to append the previewed file name

                IVsUIShellOpenDocument shellOpenDocument = GetService(typeof(SVsUIShellOpenDocument)) as IVsUIShellOpenDocument;

                // Register the Kiosk previewer
                int hr = shellOpenDocument.AddStandardPreviewer(
                    cmdLineKiosk, /*path to the exe and arguments*/
                    Resources.KioskPreviewerName, /*friendly name*/
                    0 /*fUseDDE*/, 
                    null, null, null, null, null, /*DDE related settings, if any*/
                    0 /*flags, use __VSASPFLAGS.ASP_MakeDefault to mark it default*/);
                Debug.Assert(ErrorHandler.Succeeded(hr), "Failed to add the Kiosk previewer");

                // Register the InPrivate previewer
                hr = shellOpenDocument.AddStandardPreviewer(
                    cmdLineInPrivate, /*path to the exe and arguments*/
                    Resources.InPrivatePreviewerName, /*friendly name*/
                    0 /*fUseDDE*/, 
                    null, null, null, null, null, /*DDE related settings, if any*/
                    0 /*flags, use __VSASPFLAGS.ASP_MakeDefault to mark it default*/);
                Debug.Assert(ErrorHandler.Succeeded(hr), "Failed to add the InPrivate previewer");
            }

            return VSConstants.S_OK;
        }
    }
}
