﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Globalization;
using Microsoft.VisualStudio.Shell;

namespace Microsoft.Samples.VisualStudio.BrowseWithExplorer
{
    /// <summary>
    /// This attribute declares the package will add one or more standard previewers 
    /// </summary>
    [AttributeUsage(AttributeTargets.Class, AllowMultiple = false, Inherited = true)]
    public sealed class ProvideStandardPreviewerAttribute : RegistrationAttribute
    {
        private string _previewerDescription;
        private Guid _packageGuid;

        /// <summary>
        /// Declares the package will add one or more standard previewers via calls to IVsUIShellOpenDocument.AddStandardPreviewer
        /// when the shell will call IVsPackage.ResetDefaults(PKGRF_ADDSTDPREVIEWER)
        /// </summary>
        public ProvideStandardPreviewerAttribute(string package, string previewerDescription)
        {
            this._previewerDescription = previewerDescription ?? String.Empty;
            this._packageGuid = new Guid(package);
        }

        /// <summary>
        /// Get the description of the previewers registered by this package
        /// </summary>
        public string PreviewerDescription
        {
            get { return this._previewerDescription; }
        }

        /// <summary>
        /// Get the guid of the package that will register the standard previewer
        /// </summary>
        public Guid Package
        {
            get { return this._packageGuid; }
        }

        /// <summary>
        ///     Called to register this attribute with the given context.  The context
        ///     contains the location where the registration information should be placed.
        ///     It also contains other information such as the type being registered and path information.
        /// </summary>
        public override void Register(RegistrationContext context)
        {
            // Write to the context's log what we are about to do
            context.Log.WriteLine(String.Format(CultureInfo.CurrentCulture, "Registering Standard Previewer Provider: {0} ({1})\n", Package.ToString("B"), PreviewerDescription));

            // Declare the package guid under AddStandardPreviewer, and the optional previewer description
            using (Key AddStandardPreviewers = context.CreateKey("AddStandardPreviewer"))
            {
                using (Key PackageKey = AddStandardPreviewers.CreateSubkey(Package.ToString("B")))
                {
                    PackageKey.SetValue("", PreviewerDescription);
                }
            }
        }

        /// <summary>
        /// Unregister the pacakge from providing standard previewers
        /// </summary>
        /// <param name="context"></param>
        public override void Unregister(RegistrationContext context)
        {
            // Write to the context's log what we are about to do
            context.Log.WriteLine(String.Format(CultureInfo.CurrentCulture, "Unregistering Standard Previewer Provider: {0}\n", Package.ToString("B")));

            context.RemoveKey("AddStandardPreviewer\\" + Package.ToString("B"));
        }
    }
}
